create or replace function public.return_data_model_tableau_prep(end_node text)
returns table(
	source_object	varchar
	, dependent_object	varchar
	, dependent_object_create_code	text
	, y_val	int
	, relationship	varchar
	, x_val	float
	, rel_id	int
	, rel_quality varchar
	, obj_role	varchar
)

as $function$
BEGIN
	RETURN QUERY
		-- stage 1: call upon return_data_model() to get all relevant data for the input parameter end_node
		-- Determine relationships and their quality (upstream / downstream),  assign a relation ID (rel_id)
		with prepared_model_data as
				(select
					x.source_object
					, x.dependent_object
					, x.dependent_object_create_code
					, x.source_object_stage as y_val
					, x.source_object || ' --> ' || x.dependent_object as relationship
					, x.source_object_stage
					, dense_rank() over (order by x.source_object || x.dependent_object) as rel_id
					, 'downstream' as rel_quality
				from
					return_data_model(end_node) as x
					
				union all
				
				select
					y.dependent_object
					, y.source_object
					, '' as dependent_object_create_code
					, y.dependent_object_stage
					, y.source_object || ' --> ' || y.dependent_object as relationship
					, y.dependent_object_stage
					, dense_rank() over (order by y.source_object || y.dependent_object) as rel_id
					, 'upstream' as rel_quality
				from
					return_data_model(end_node) as y),
					
		-- stage 2: determine a preliminary x-value for the display of data model elements in the network visualization in Tableau
			ranked_model_data as
				(select
					prepared_model_data.*
					, dense_rank() over (partition by prepared_model_data.source_object_stage order by prepared_model_data.source_object) as x_val
				from
					prepared_model_data)
				
		-- ###################################################
		-- MAIN QUERY
		-- ###################################################
		
		select
			z.source_object
			, z.dependent_object
			, z.dependent_object_create_code
			, z.y_val
			, z.relationship::varchar
			-- add a little additional offset to the x value per element
			-- this allows for better alignment of the nodes between elements in the network visualization in Tableau
			, (z.x_val + 0.2 * z.y_val)::FLOAT as x_val
			, z.rel_id::INT
			, z.rel_quality::VARCHAR
			, case
				when sum(case when z.rel_quality='upstream' then 1 END) over (partition by z.source_object) IS NULL then 'root_node'
				when sum(case when z.rel_quality='downstream' then 1 END) over (partition by z.source_object) IS NULL then 'end_node'
				ELSE 'intermediary'
			END::VARCHAR as obj_role
		from
			ranked_model_data as z;
END;

$function$ language plpgsql